\name{coldmap}
\alias{coldmap}
\title{Cooler heatmap}
\description{Heatmap for multiple, large data matrices}
\usage{
coldmap(..., clust=NULL,
  dlab=NULL,
  clab = NULL,
	alt.colnames=NULL,
  clab.height = 2,
  col.clab=NULL,
  cex.clab=.8,
  rlab = NULL,
  rlab.width = NULL,

  rannot = NULL,
  rannot.width=1.5,
  
  col=rb.colors,
  saturation=.666,
  downsampling = NULL,
  rcenter = FALSE,
  ccenter = TRUE,

  wa = NULL, wb = NULL,
  rw = 1, rwa = NULL, rwb = NULL,
  chang = 0.0,
  rhang = 0.05,
  gutter=NULL, proportional=TRUE,
  rpwcorr=TRUE,
  cpwcorr=FALSE,
  rdw = 3, cdw = 3,
  rticks=NULL, cticks=NULL,
	rorder=NULL,
  corder=NULL,

  cplot = NULL,
 	cplot.band=c(0),
 	cplot.height = 2,
 	cplot.gutter=0.4,
 	cplot.panel=cplot.points,
  rclist = NULL
  )
}
\arguments{
\item{...}{Data matrices}
\item{clust}{clustering output (of previous call to \code{coldmap} or \code{nclust2})}
\item{dlab}{dataset labels}
\item{clab}{column labels: if it is a vector of integers, they are interpreted
as position in the sorted heatmap. If it is a vectors of strings, they
will be matched against the union of column names.}
\item{clab.height}{height of the column label panel, in cm.}
\item{cex.clab}{Relative font size for \code{clab}}
\item{col.clab}{colors for \code{clab}}
\item{rlab}{A list, each element corresponding to a dataset, of row
labels to be printed. As in \code{clab}, if it is a vector of integers,
it is interpreted as position in the tree.}
\item{rlab.width}{The width, in cm, of row label panels.}
\item{rannot}{Additional row annotations, as a list of matrices of
charaters defining the color of the cells. The number of rows should
match the number of rows in respective datasets.}
\item{col}{colors as described in \link{heat.colors}. This package
  defines \code{rb.colors} (red-white-blue), \code{rg.colors} (red-black-green)
  and \code{bw.colors} (gray levels)}
\item{saturation}{Degree of color saturation. Negative value invert the
  color gradient.}
\item{downsampling}{Downsampling windows for displaying heatmpas.
  An integer vector \code{c(a,b)}
  means that each output pixel is an average of
  non-overlapping block of size a rows and b columns.
  The default produces a heatmap of approximately
  200-by-400 pixels, per dataset.}
\item{rcenter,ccenter}{Mean centering of the rows and columns of each
dataset. These options are used only for displaying the images (not the clustering
algorithm)}
\item{wa,wb}{Zoom window range, where \code{wa < wb}. The numbers are
position in the global tree. The default is all are shown.}
\item{rw}{Choose dataset for row zoom in (an integer index).}
\item{rwa,rwb}{Row zoom range for dataset chosen by \code{rw}. Currently,
setting these to non-NULL implies \code{proportional=FALSE}.}
\item{rhang,chang}{Row and column "hang": the length of the stem connecting
a terminal node to its parent (as a proportion of the whole tree height).
A negative value causes all terminal stems to be drawn to the same height,
at a location specified by the magnitude of the value.}
\item{gutter}{gap between heatmaps (measured in number of samples). By
default it is one twentieth of the average sample size.}
\item{proportional}{Draw the each heatmap with a height proportional
to the sample size.}
\item{rpwcorr,cpwcorr}{Save row and/or column pairwise correlation matrix.
They can be found as \code{$pwcorr} object inside \code{rclust}
and \code{cclust}. Beware that \code{cpwcorr} can produce extremely
large matrix (the number of columns squared).
}
\item{rdw,cdw}{Row (column) dendrogram width (height) in centimeters.
This is fixed size. Resizing plotting window only resize the heatmaps.}
\item{rannot.width}{Width of row annotation, in cm}
\item{rgap}{Gap between heatmaps, used only when \code{proportional=FALSE}}
\item{rticks,cticks}{Tick marks intervals (an integer).
The default is automatic. If \code{"l"} is specified for \code{rticks}, the
row labels will be printed for each row without regard of overcrowding. This
should be used only for small datasets.}
\item{rorder}{List of vector of row names (or a vector if only a single
	dataset is used). The heatmap will be displayed only
for these variables, in this given order. No row dendrogram will
be drawn.}
\item{corder}{Vector of column names. The heatmap will be displayed only
for these variables, in this given order. No column dendrogram will
be drawn.}
\item{cplot}{Matrix of continuous values to be plotted above the
heatmaps. Multiple rows correspond to multiple plots. Each column of
the \code{cplot} matrix corresponds to the union of the column of the
data matrix. The data is shown according to the cplot matrix column
ordering. No automatic lookup is done to match the input data label yet.
NOTE: this is a bug that should be fixed in the future.}
\item{cplot.band}{Draw red horizontal red lines at the specified values. Useful
for showing cutoffs of the values.}
\item{cplot.height}{Height of plots in cm}
\item{cplot.gutter}{Margins between plots when multiple rows are supplied}
\item{cplot.panel}{the function which plots the points in the top
  panel. Takes only \code{x} and \code{y} as arguments. See the source
  of the default \code{cplot.points} for an example.}
\item{rclist}{Use custom row clusters. It is a list of clustering output in \code{hclust}
format. The member of the list should match the number of datasets. Tree height values
are assumed to be decreasing towards the leaf (as in standard \code{hclust}).}
}
\details{
\code{coldmap} shows heatmaps of multiple datasets, with rows and
columns ordered according to hierarchical clustering. The dendrograms
are shown on the margins.
}
\value{
A list of \code{cclust} and \code{rclust}, for column and row
clustering, respectively. (Note that the columns of the matrix
are on the vertical axis of the plot, following the convention
in \link{image}).
}
\seealso{nclust,nclust2,nclust.mmat,pwmap}
\examples{

# load data

load(url("http://lausanne.isb-sib.ch/~pwirapat/amac2009/packages/TGIF1.rda"))
load(url("http://lausanne.isb-sib.ch/~pwirapat/amac2009/packages/JRH2.rda"))

# make the clustering (will take a few minutes for these datasets)

clust <- nclust2( TGIF1$gex, JRH2$gex )

# show the heatmaps

coldmap( TGIF1$gex, JRH2$gex, clust = clust, dlab=c("TGIF1","JRH2") )

# look at some genes (it's a list of several character vectors)
data(bcmodules)

bcmodules   # have a look

coldmap( TGIF1$gex, JRH2$gex, clust = clust, dlab=c("TGIF1","JRH2"),
  clab = unlist(bcmodules) )

# color the genes according the grouping in bcmodules
modulecolor <-  mcol <- c("red","darkgreen","orange","blue","purple","brown","black")
genecolor <- unlist(mapply(function(u,v)rep(v,length(u)),bcmodules,modulecolor))

coldmap( TGIF1$gex, JRH2$gex, clust = clust, dlab=c("TGIF1","JRH2"),
  clab = unlist(bcmodules), col.clab=genecolor )

# show additional row annotations

coldmap( TGIF1$gex, JRH2$gex, clust = clust, dlab=c("TGIF1","JRH2"),
  clab = unlist(bcmodules), col.clab=genecolor,
  rannot = list(
  cbind( "ER status" = c("orange","green")[1+TGIF1$clin$er], "tumor grade" = c("blue","gray","red")[TGIF1$clin$grade] ),
  cbind(c("orange","green")[1+JRH2$clin$er], c("blue","gray","red")[JRH2$clin$grade] )
  )
)

# additional top panel genes statistics
AVG <- rbind(TGIF1 = colMeans(TGIF1$gex),
             JRH2 = colMeans(JRH2$gex))
cplot.func <- function(x, y) {
  points(x, y, cex=0.1, col=rgb(0,0,0,0.2))
  lines(lowess(x, y, f=0.1), lwd=1, col="red")
}
coldmap( TGIF1$gex, JRH2$gex, clust = clust, dlab=c("TGIF1","JRH2"),
  clab = unlist(bcmodules), col.clab=genecolor,
  rannot = list(
  cbind( "ER status" = c("orange","green")[1+TGIF1$clin$er], "tumor grade" = c("blue","gray","red")[TGIF1$clin$grade] ),
  cbind(c("orange","green")[1+JRH2$clin$er], c("blue","gray","red")[JRH2$clin$grade] )
  ),
  cplot=AVG, cplot.band=c(), cplot.panel=cplot.func
)

}
